package Chandra::Tools::ECF;

use 5.008;
use strict;
use warnings;

use Astro::FITS::CFITSIO::Simple qw(rdfits);
use PDL;
use Carp;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Chandra::Tools::ECF ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw( );
our $VERSION = '0.01';
our @FIELDS = qw(radius radius_smin radius_smax y z sma smb pa);
(our $Circular_File   = __FILE__) =~ s|/ECF\.pm|/circular_ECF.fits|;
(our $Elliptical_File = __FILE__) =~ s|/ECF\.pm|/elliptical_ECF.fits|;

##***************************************************************************
sub new {
##***************************************************************************
    my $classname = shift;
    my $self = { };
    bless ($self, $classname);
    local $_;
    my $file = shift;
    
    if (defined $file) {
	eval { $self->{data} = { rdfits($file) } };
	if ($@) {
	    croak "Chandra::Tools::ECF::new - can't read '$file'\n";
	}
	my $d = $self->{data};

	foreach (@FIELDS) {
	    next unless defined $d->{$_};
	    $d->{$_}->reshape( $d->{ecf}->nelem,
			       $d->{theta}->nelem,
			       $d->{phi}->nelem,
			       $d->{energy}->nelem);
	}
    } else {
	croak "Chandra::Tools::ECF - need an ECF file\n";
    }

    # Put in a phantom entry for phi to ensure correct wrap in nearest
    # neighbor search (e.g. for phi=355)
    $self->{data}{phi} = $self->{data}{phi}->append(360.0);

    return $self;
}

##***************************************************************************
sub ECF {
##***************************************************************************
    my $self = shift;
    my %par = ( @_);
    local $_;
    
    my $d = $self->{data};
    my %i;
    my %out;

    # Find nearest 4-d grid point
    for (qw(ecf theta phi energy)) {
	croak "Chandra::Tools::ECF - need '$_' parameter" unless defined $par{$_};
	$i{$_} = ( minmaximum( abs($d->{$_} - $par{$_})) )[2]->sclr;
    }

    # Last value of phi (360) wraps to zero
    $i{phi} = $i{phi} % ($self->{data}{phi}->nelem - 1);

    # Define output value(s) at nearest grid point.  For circular, only
    # radius will be def'd, while for ellipse the latter 5 will exist.
    foreach (@FIELDS) {
	next unless defined $d->{$_};
	$out{$_} = $d->{$_}->at($i{ecf}, $i{theta}, $i{phi}, $i{energy});
    }
    
    return %out;
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Chandra::Tools::ECF - Perl extension for interpolating the Chandra Enclosed Counts
Fraction files

=head1 SYNOPSIS

  use Chandra::Tools::ECF;
  
  # Read ECF data from a user-specified file
  my $ecf      = Chandra::Tools::ECF->new("my_ECF_file.fits");

  # Read ECF data from standard files included in package
  my $circ_ecf = Chandra::Tools::ECF->new( $Chandra::Tools::ECF::Circular_File );
  my $ell_ecf  = Chandra::Tools::ECF->new( $Chandra::Tools::ECF::Elliptical_File );

  # Specify these four values (all required)
  %spec = (ecf   => 0.9,
	   theta => 5,
	   phi   => 180,   # This must be given 0 <= phi < 360
	   energy => 1.3);

  %ecf  = $ecf->ECF( %spec );
  %circ = $circ_ecf->ECF( %spec );  
  %ell  = $ell_ecf->ECF( %spec );

  print "Radius: $circ{radius}  Smin: $circ{radius_smin}  Smax: $circ{radius_smax}\n";
  print "sma:$ell{sma} smb:$ell{smb} y:$ell{y} z:$ell{z} pa:$ell{pa} \n";

=head1 DESCRIPTION

Chandra::Tools::ECF is provides a simple method to access the Chandra HRMA Enclosed
Counts Fraction (ECF) data.  Currently the tool uses nearest neighbor interpolation 
of the provided position to the ECF grid.

=head1 EXPORT

None.

=head1 SEE ALSO

http://cxc.harvard.edu/cal/Hrma/psf/

=head1 AUTHOR

Tom Aldcroft, E<lt>taldcroft at cfaE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Tom Aldcroft

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.3 or,
at your option, any later version of Perl 5 you may have available.

=cut
