package Yaxx;

use Data::ParseTable qw(parse_table);
use Ska::HashTable;
use Carp;
use Data::Dumper;
use PDL;

1; 

##****************************************************************************
sub get_object_data {
##****************************************************************************
    my $file = shift;
    my %count;
    my @data_out;		# Output data

    my %yaxx_fields = ( 'x'   => 'X',
			'y'   => 'Y',
			ra    => 'ra',
			dec   => 'dec',
			src   => 'srcid',
			ccdid => 'ccdid',
			detector => 'detector',
			obsid => 'obsid',
			rad   => 'rad',
			# component => 'srcid',
			object => 'object',
		      );

    local $_;

    # If the supplied file name is empty just return
    return unless (defined $file and $file =~ /\S/);

    croak "ERROR - can't find object_list file $file\n" unless (-r $file);

    message(1, "Reading object list from $file\n");
    # Read the object data in as an array of hashes
    my @data_in = @{ parse_table($file, {comment => qr/\s*#/}) }; # This reads FITS, RDB, HTML, or ASCII

    # Make sure we got some data
    unless (@data_in) {
	message(0, "ERROR - object list file '$file' is empty");
	confess $Yaxx::FATAL_ERROR;
    }	

    # There must be an 'obsid' column
    if (not defined $data_in[0]->{obsid}) {
	message(0, "ERROR - need an 'obsid' column in object list file '$file'\n");
	confess $Yaxx::FATAL_ERROR;
    }		  

    message(5, "Object list: data_in = ", Dumper \@data_in);

    # If none of the columns ra,dec,x,y exist, then bring in src2 file data
    unless (grep /^(ra|dec|x|y)$/i, keys %{$data_in[0]}) {
	@data_in = include_src_file_data(@data_in); # Expand any refs to detect file data
	message(5, "After including source file: data_in = ", Dumper \@data_in);
    }


    # Process each object data entry, making sure there is at least an obsid and srcid
    foreach my $data_in (@data_in) {
	delete $data_in->{$_} foreach (grep {ref $data_in->{$_}} keys %{$data_in});

	# The key/val pairs in $data_out go directly into Yaxx object, so only
	# certain allowed values are copied in.  But put all data are in the "source_data"
	# attribute for use in user-defined fit scripts etc
	my $data = { source_data => { %{$data_in} } };

	# Copy certain data into Yaxx attributes as spec'd in %yaxx_fields.
	# For convenience make the input matching case-independent so user could
	# have "RA Dec" in their input file and it will still fly
	foreach (keys %{$data_in}) {
	    my $yaxx_key = $yaxx_fields{ lc $_ };
	    $data->{$yaxx_key} = $data_in->{$_} if (defined $yaxx_key);
	}

	# Generate a srcid if not provided
	$data->{srcid} = ++$count{ $data->{obsid} } unless defined $data->{srcid};

	push @data_out, $data;
    }

    return @data_out;
}

##****************************************************************************
sub include_src_file_data {
#
# Grab object data from file specified in the 'src_file' column of the objlist 
# input file.  This data will typically be a detect (wavdetect or celldetect)
# FITS file.  
#
# Output: array of merged object and source (detect) data
#
##****************************************************************************
    my @obj_data = @_;		# Object data from "objlist" file (array of hash-refs)
    my @obj_data_out;
    my %src_table_for;		# Hash of src (detect) file data based on obsid
    local $_;
    my @files;

    foreach my $obj_data (@obj_data) {
	my $obsid = $obj_data->{obsid};
	if (not defined $src_table_for{$obsid}) {
	    my $input_dir = get_input_dir($cfg{input_dir}, $obsid);

	    foreach (split ' ',$cfg{src2_glob}) {
		last if (@files = glob($input_dir."/$_") and not $!);
	    }

	    # Make sure we got one and only one event file
	    if ($! or @files > 1 or @files == 0) {
		my $err = (@files > 1) ? 'too many' : 'no';
		message(0, "ERROR - $err src2 files matching '".$cfg{src2_glob}."' for obsid $obsid\n");
		confess $Yaxx::FATAL_ERROR;
	    } 

	    $src_table_for{$obsid} = Ska::HashTable->new($files[0]); ;
	}

	# Rewind wavdetect table (could be at end from previous objects with same obsid)
	$src_table_for{$obsid}->rewind;

	# Combine $obj_data (obj_list file) and $data (from FITS
	# source file), with $obj_data taking precedence.  Lowercase all keys
	# to prevent possible errors from capitalized col names (e.g. RA). 
	my $src_col = $cfg{src_column_name};
	my $net_cts_col = $cfg{src_net_cts_name};
	while (my %data = $src_table_for{$obsid}->row()) {
	    if (not defined $obj_data->{src}
		or $obj_data->{src} !~ /^\d+$/
		or $obj_data->{src} == $data{$src_col}) {

		# Skip source if not enough net counts
		next if (defined $data{$net_cts_col} and $data{$net_cts_col} < $cfg{min_counts});

		# Start with values from src (detect) file
		my %obj_src_data = map { lc($_) => $data{$_} } keys %data;

		# Override with values from object (objlist) file, but only if the new
		# value is non-trivial or if the existing value (from src) is undefined
		foreach (keys %{$obj_data}) {
		    my $lc_key = lc($_);
		    if ($obj_data->{$_} =~ /\S/ or not defined $obj_src_data{$lc_key}) {
			$obj_src_data{$lc_key} = $obj_data->{$_};
		    }
		}
		push @obj_data_out, { %obj_src_data };
	    }
	}
    }
    return @obj_data_out;
}

