%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Id: sherpa_plotfns_2.3.sl,v 1.1.1.1 2004/02/20 16:01:30 aldcroft Exp $
%
% File:
%   sherpa_plotfns.sl
%
% Thread:
%   http://cxc.harvard.edu/ciao/threads/sherpa_slangplot/
%   http://ledas-cxc.star.le.ac.uk/ciao/threads/sherpa_slangplot/
%
% Aim:
%   Augment the plotting capabilities of Sherpa by using S-Lang to call
%   ChIPS functions.
%
% Functions:
%   lpfr()
%     - "lplot 2 fit residuals" + customizations
% 
%   lpfd()
%     - "lplot 2 fit delchi" + customizations
%
% Notes:
%   If a function is called to plot a dataset that doesn't exist (eg lpfd
%   when nothing has been loaded/fit), then Sherpa will hang; a control-c
%   is needed to exit. This is due to a limitation of the current S-lang
%   interface to Sherpa.
%
%   If the ``sherpa_utils'' S-Lang package has been loaded (available from
%   http://cxc.harvard.edu/ciao/download_scripts.html and
%   http://ledas-cxc.star.le.ac.uk/ciao/download_scripts.html) then
%   advantage is taken of these routines:
%     plot titles will show the model name rather than the current time
%     plots will fail to display unless a model exists (partly helping out
%     with the first note)
%
% History:
%   2002/04/25 - added lpfd()
%   2002/06/05 - take advantage of sherpa_utils functions if they exist
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% lpfr() and lpfd() are defined at the end of this file. 
% Here we define several routines that are used by these
% functions (and actually do all the work). They are defined to be static
% so that they are only visible from within this file.
%

% execute a Sherpa command, exiting on error
private define seval( command ) {
  variable retval = sherpa_eval( command );
  if ( retval != 0 )
    error( "\nError: unable to execute the following Sherpa command\n" +
           command + "\n" );
} % seval()

% execute a ChIPS command, exiting on error
private define ceval( command ) {
  variable retval = chips_eval( command );
  if ( retval != 0 )
    error( "\nError: unable to execute the following ChIPS command\n" +
           command + "\n" );
} % ceval()

%
% Usage:
%   _plot_fit_and( type, n )
%
% Aim:
%   Plots fit and the second option, then hacks around to customize the plots.
%   The major change is to ensure that the y axis for the second plot 
%   is ALWAYS linear, whatever the user has set 'set plot ...' to. We also
%   change the layout of the two graphs so that the fit window is larger,
%   and that there's no gap between the two windows.
%
%   If the sherpa_utils routines are available then we change the plot title
%   to be the "name" of the model (if 'source = xspowerlaw[pl] * xsphabs[gal]'
%   then the name is 'pl * gal') rather than the current time.
%
% Note:
%   To see the change made by this routine you have to call "redraw" after
%   returning from this function. It could be moved inside this function, but
%   that would mean more "flashing" by the graph if you wish to apply further
%   change the plot(s).
%
% Examples:
%   _plot_fit_and( "resid", 1 );
%   () = chips_eval( "redraw" );
%
%   _plot_fit_and( "delchi", 3 );
%   () = chips_eval( "redraw" );
%

% we have to have the variable outside the function definition or
% else the eval() statement below would not be able to access it.
% I'd prefer it if the variable was not placed into the global
% namespace, but I'll leave it as is for now
variable _modelname;

static define _plot_fit_and () {

  % check how the routine was called
  variable usage_str = "Usage: _plot_fit_and( type, n )\n";
  if ( _NARGS != 2 ) {
    __pop_args( _NARGS );
    message(usage_str);
    return 0;
  }

  % NOTE: we have NO error checking on the input values
  variable type, n;
  ( type, n ) = ();

  % if the Sherpa Utils package has been loaded, find
  % out the name of the model - used in the title but it also
  % ensures that the model exists
  _modelname = NULL;
  if ( is_defined("get_modelname") == 2 ) {
    eval( "_modelname = get_modelname(" + string(n) + ");" );
    if ( _modelname == NULL ) {
      message( "\nError: there is no model defined for dataset number " + 
        string(n) + "\n" );
      return 0;
    }
  }

  % plot the data then customise its look
  seval( "lplot 2 fit " + string(n) + " " + type + " " + string(n) );

  % make the second plot smaller (and remove the gap between the plots)
  ceval( "d 1 location 0.15 0.9 0.4 0.9" );
  ceval( "d 2 location 0.15 0.9 0.1 0.4" );
  
  % remove the x axis values from the first plot
  ceval( "d 1 tickvals x off" );

  % force the second plot to have a linear y axis
  ceval( "d 2 linear y" );

  % change the symbol & size of the residual plot
  ceval( "c 1 symbol bigpoint" );
  ceval( "symbol size 1" );

  % Change the title: new title depends on what routines are available
  variable titlestring;
  if ( _modelname != NULL ) {
    titlestring = "Model = " + _modelname;
  } else {
    % add a title including the current time (using the S-Lang time function)
    titlestring = time;
  }
  ceval( "title '" + titlestring + "'" );

  % NOTE: to see these changes the calling routine/user must
  % call ChIPS' redraw command.
  %%ceval( "redraw" );

  return 1;

} % _plot_fit_and()

%
% Usage:
%   n = _get_dataset_num( name, arguments... )
%
% Aim:
%   Evaluates the input arguments to the function (currently
%   lpfr and lpfd) and returns the value of the dataset to use.
%   If no argument is given, default to using the first dataset.
%
%   There's minimal error checking, since it's not easy in CIAO 2.2.1
%
% Examples:
%   % check how the routine was called
%   variable args = __pop_args (_NARGS);
%   variable n = _get_dataset_num( "lpfr", __push_args(args) );
%   
static define _get_dataset_num () {

  % parse the arguments - this has the side effect of clearing the stack
  variable n, funcname;
  switch ( _NARGS )
  { case 0: error("Internal error: _get_dataset_num() called with no arguments!!!"); }
  { case 1: n = 1; funcname = (); }
  { case 2: ( funcname, n ) = (); }
  { 
    % too many arguments
    variable args = __pop_args( _NARGS );
    funcname = args[0].value;

    % dummy value for n, needed since 'or' (if statement below)
    % does not "short circuit" like || in C
    n = 0;
  }

  % is there a problem?
  variable usage_str = "Usage: " + funcname +
	"( [n] )\nIf supplied, n must be >= 1.";
  if ( _NARGS > 2 or n < 1 ) {
    message(usage_str);
    return NULL;
  }

  % success
  return n;

} % _get_dataset_num()

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% public routines
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Usage:
%   lpfr
%   lpfr()
%   lprf(n)
%
% Aim:
%   Plots fit and residual, then hacks around to change the look slightly.
%   see _plot_fit_and() for details of the changes.
%
% If supplied without an argument, plots the first dataset 
% otherwise plots the given dataset number (n must be an integer >= 1)
% There's no error checking that there is actually a dataset/fit for
% number n (not easily done in CIAO 2.2.1)
%
% If the sherpa_utils routines are available then we also check
% that a model exists for this particular dataset number
% (since a model is needed for plotting the fit)
%
public define lpfr () {

  % delegate argument handling to _get_dataset_num(), which is why
  % we have to pass it all the arguments (could be neater)
  variable args = __pop_args (_NARGS);
  variable n = _get_dataset_num( _function_name, __push_args(args) );
  if ( n == NULL ) return;

  % all the hard work is delegated to another function ...
  variable retval = _plot_fit_and( "resid", n );

  % we want the user to see the changes, so
  if ( retval == 1 ) ceval( "redraw" );

} % lpfr()

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Usage:
%   lpfd
%   lpfd()
%   lpfd(n)
%
% Aim:
%   Plots fit and delchi, then hacks around to change the look slightly.
%   see _plot_fit_and() for details of the changes.
%
%   Same calling sequence as lpfr()
%
public define lpfd () {

  % delegate argument handling to _get_dataset_num(), which is why
  % we have to pass it all the arguments (could be neater)
  variable args = __pop_args (_NARGS);
  variable n = _get_dataset_num( _function_name, __push_args(args) );
  if ( n == NULL ) return;

  % all the hard work is delegated to another function ...
  variable retval = _plot_fit_and( "delchi", n );

  % we want the user to see the changes, so
  if ( retval == 1 ) ceval( "redraw" );

} % lpfd()

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
